/*   Defines to make python-dmidecode work with more Python versions
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
 *
 *   For the avoidance of doubt the "preferred form" of this code is one which
 *   is in an open unpatent encumbered format. Where cryptographic key signing
 *   forms part of the process of creating an executable the information
 *   including keys needed to generate an equivalently functional executable
 *   are deemed to be part of the source code.
 */


#ifndef _COMPAT_H
#define _COMPAT_H

// Missing in Python 2.3
#ifndef Py_RETURN_TRUE
#define Py_RETURN_TRUE return Py_INCREF(Py_True), Py_True
#endif

// Missing in Python 2.3
#ifndef Py_RETURN_FALSE
#define Py_RETURN_FALSE return Py_INCREF(Py_False), Py_False
#endif

// Missing in Python 2.3
#ifndef Py_RETURN_NONE
#define Py_RETURN_NONE return Py_INCREF(Py_None), Py_None
#endif

// Python 2 vs Python 3 compat
#if PY_MAJOR_VERSION >= 3
#define IS_PY3K
#define MODINITERROR return NULL
#define PYNUMBER_FROMLONG PyLong_FromLong
#define PYTEXT_FROMSTRING PyUnicode_FromString
#else
#include <bytesobject.h>
#define MODINITERROR return
#define PYNUMBER_FROMLONG PyInt_FromLong
#define PYTEXT_FROMSTRING PyString_FromString
#define PyCapsule_New(pointer, name, destructor) \
	    (PyCObject_FromVoidPtr(pointer, destructor))
#endif

#endif
